// naumann@stce.rwth-aachen.de

#include<cmath>
#include<iostream>

template<typename T, typename PT>
T f(T &x, const PT &p) {
   return pow(x,2)-p;
}

template<typename T, typename PT>
void f_t(T &x, T& x_t, const PT &p, const PT &p_t, T &r, T &r_t) {
   r_t=2*x*x_t-p_t;
   r=pow(x,2)-p;
}

template<typename T, typename PT>
T dfdx(T &x, const PT &) {
  return 2*x;
}

template<typename T, typename PT>
void dfdx_t(T &x, T &x_t, const PT &, const PT &, T &drdx, T& drdx_t) {
  drdx_t=2*x_t;
  drdx=2*x;
}

template<typename T, typename PT>
void newton_t(T &x, T &x_t, const T &p, const T &p_t, const PT &eps, const unsigned int maxit) {
  unsigned int it=0;
  T y,y_t;
  f_t(x,x_t,p,p_t,y,y_t);
  do {
    T dydx,dydx_t;
    dfdx_t(x,x_t,p,p_t,dydx,dydx_t);
    x_t-=y_t/dydx-y*dydx_t/pow(dydx,2);
    x-=y/dydx;
    f_t(x,x_t,p,p_t,y,y_t);
    if (++it==maxit) break;
  } while(fabs(y)>eps);
}
